@@ -6,10 +6,11 @@ import unittest
 from datetime import datetime, date
 from os.path import join as path_join
 from typing import Any, Type, Union
+from copy import deepcopy
 
 from marshmallow import Schema
 from mongoengine.base import TopLevelDocumentMetaclass
-from snuff_utils.string_functions import f
+from mongoengine import Document
 
 
 class CommonTestCase(unittest.TestCase):
@@ -26,38 +27,40 @@ class CommonTestCase(unittest.TestCase):
     test_data_file_name = None
     _base_dir = None
     models_map = None
+    counter_map = {}
+    template_url = None
+    user_for_auth = None
+    password_for_auth = None
 
     @classmethod
-    def setUpClass(cls, *args):
+    def setUpClass(cls, create_app, config, db, *args):
         """
         Start Flask app end check test database name in current db
 
-        Please pass three arguments:
-            args[0] - Function for create flask app
-            args[1] - Test config for flask app
-            args[2] - DB
+        :param create_app: Function for create Flask app
+        :param config: Config for Flask app
+        :param db: Database for tests
         """
-        if len(args) < 3:
-            raise AssertionError('Please pass three arguments')
-        create_flask_func = args[0]
-        config = args[1]
-        cls.db = args[2]
-
         # Start flask app
-        app = create_flask_func(config)
+        app = create_app(config)
         cls.client = app.test_client()
         cls.app_context = app.app_context()
         cls.app_context.push()
+        cls.db = db
+        cls._prepare_database(config=config, db=db)
 
-        # Check test database name in current db
-        cls.test_db_name = app.config['TEST_DB_NAME']
-        dev_db_name = app.config['DEV_DB_NAME']
-        if cls.test_db_name in cls.db.connection.list_database_names():
-            cls.db.connection.drop_database(cls.test_db_name)
+    @classmethod
+    def _prepare_database(cls, config, db):
+        """Prepare test database"""
+        cls.test_db_name = db.connection._MongoClient__default_database_name
+
+        # Check test database name in test db
+        if cls.test_db_name in db.connection.list_database_names():
+            db.connection.drop_database(cls.test_db_name)
 
         # Create all collections from dev db
-        for collection_name in cls.db.connection[dev_db_name].list_collection_names():
-            cls.db.connection[cls.test_db_name].create_collection(name=collection_name)
+        for collection_name in list(set([item._get_collection_name() for item in cls.models_map.values()])):
+            db.connection[cls.test_db_name].create_collection(name=collection_name)
 
     @classmethod
     def tearDownClass(cls):
@@ -72,16 +75,27 @@ class CommonTestCase(unittest.TestCase):
             raise AssertionError("Not found request method!")
 
     @classmethod
-    def create_user(cls, username='unit@test.ru', password='test_pass', first_name='test', **other_data):
-        user_data = {"email": username, "first_name": first_name, "phone": '+1234567890', **other_data}
-        if not (auth_user := cls.user_model.objects(**user_data).first()):
-            auth_user = cls.user_model.objects.create(**user_data)
-        auth_user.set_password(password)
-        auth_user.save()
-        return auth_user
+    def create_user(cls, key: str = 'user', password: str = 'test_pass', **other_data) -> Document:
+        """
+        Create user and set password
 
-    def auth(self, auth_url: str = '/api/login/', username: str = 'unit@test.ru', password: str = 'test_pass',
-             blocked_user: bool = False, not_found_user: bool = False):
+        :param key: Key data for user. (In *.json file)
+        :param password: User password
+        :param other_data: Other fields
+
+        :return Created user
+        """
+        user = cls.generate_test_data(key=key, **other_data)
+        user.set_password(password)
+        user.save()
+        return user
+
+    def auth(self,
+             username: str = None,
+             password: str = None,
+             auth_url: str = '/api/login/',
+             blocked_user: bool = False,
+             not_found_user: bool = False):
         """
         Authorization function.
 
@@ -93,6 +107,10 @@ class CommonTestCase(unittest.TestCase):
         """
         self.client.cookie_jar.clear()
         self.authorized = False
+
+        username = username if username else self.user_for_auth.email
+        password = password if password else self.password_for_auth
+
         status_code = 400 if blocked_user or not_found_user else 200
         json_request = {"email": username, "password": password}
         json_response = self._send_request(url=auth_url, params=json_request, expected_status_code=status_code,
@@ -149,14 +167,14 @@ class CommonTestCase(unittest.TestCase):
         :param status_code Expected status code
         :param many
         """
-        if many:
-            not_found_id = [not_found_id]
         if id_in_data:
+            if many:
+                not_found_id = [not_found_id]
             request_data = {field: not_found_id}
             url = self.url
         else:
             request_data = {}
-            url = '/'.join(self.url.split('/')[:-2] + [not_found_id])
+            url = self.template_url.format(**{field: not_found_id})
         json_response = self._send_request(url=url, params=request_data, expected_status_code=status_code)
         if many:
             return self.assertIn(f'Could not find document.', json_response['errors'][field])
@@ -170,8 +188,10 @@ class CommonTestCase(unittest.TestCase):
         """
         for role in role_keys:
             self.client.cookie_jar.clear()
-            user = self.create_user(username=f'{role}@forbidden.com', password='pass', role=role)
-            self.auth(username=user.email, password='pass')
+            email = f'{role}@forbidden.com'
+            password = 'pass'
+            self.create_user(email=email, password=password, role=role)
+            self.auth(username=email, password=password)
             json_response = self._send_request(expected_status_code=403)
             self.assertIn('errors', json_response)
             self.assertIn("role", json_response['errors'])
@@ -404,11 +424,11 @@ class CommonTestCase(unittest.TestCase):
             else:
                 raise AssertionError(f'File not found! {path}')
 
-        data = get_data_from_file()
-        data.update(other_data)
-        for i in range(1, count_create + 1):
-            create_data = {key: f(value, i=i) if isinstance(value, str) else value for key, value in data.items()}
-            instance = model.objects.create(**create_data)
+        raw_data = get_data_from_file()
+        raw_data.update(other_data)
+        for i in range(count_create):
+            data = cls._counter_data(key_object=key, raw_data=raw_data)
+            instance = model.objects.create(**data)
             instances.append(instance)
             cls.test_docs.append(instance)
         if not many or count_create == 1:
@@ -448,6 +468,19 @@ class CommonTestCase(unittest.TestCase):
 
         return bad_data
 
+    @classmethod
+    def _counter_data(cls, key_object, raw_data):
+        new_data = {}
+        for field, value in deepcopy(raw_data).items():
+            if isinstance(value, str) and '{i}' in value:
+                cls.counter_map.setdefault(key_object, {})
+                last_count = cls.counter_map[key_object].get(field, 0)
+                cls.counter_map[key_object][field] = last_count + 1
+                new_data[field] = value.format(i=cls.counter_map[key_object][field])
+            else:
+                new_data[field] = value
+        return new_data
+
     def _send_request(self,
                       url: str = None,
                       params: dict = None,
